#
# RPM specifications file for the GNU Compiler Collection
#
# Note: This specifications file requires rpm version 3.0 or newer.
#

%define GCC_VERSION 2.95.2
%define STDC_VERSION 2.10.0
%define target_alias %{_target_cpu}-unknown-linux-gnu
%define OBJDIR $RPM_BUILD_DIR/obj-$RPM_ARCH-linux

Summary: GNU Compiler Collection - The C Compiler
Name: gcc
Version: %{GCC_VERSION}
Release: 1nw7
Copyright: GPL
Group: Development/Languages
Source0: ftp://gcc.gnu.org/pub/gcc/releases/gcc-%{GCC_VERSION}/gcc-%{GCC_VERSION}.tar.gz
Source1: gcc-libstdc++-compat.tar.gz
# libobjc patch based on HJ Lu's threading patch for libobjc.
#   ftp://ftp.varesearch.com/pub/support/hjl/gcc
Patch0: gcc-2.95.2-libobjc.patch
# Java patch based on Bryce McKinlay's updated patch from:
#  http://waitaki.ctago.ac.nz/~bryce/gcj/
Patch1: gcc-2.95.2-java3.patch
# ARM patch based on my own work and Phil Blundell's patch from:
#  ftp://ftp.netwinder.org/users/p/philb/
Patch2: gcc-2.95.2-arm6.patch
# libio patch from Jakub Jelinek  <jakub@redhat.com>
# fixes some problems with between glibc 2.1.x and glibc 2.1.9x/2.2
# take from compat-egcs-6.2-1.1.2.9.src.rpm in Red Hat 7.0
Patch3: egcs-libio.patch
Buildroot: /var/tmp/gcc-%{GCC_VERSION}-root
Url: http://gcc.gnu.org/
Requires: cpp = %{GCC_VERSION}
Requires: binutils >= 2.9.5.0.19
Requires: libfloat
Obsoletes: egcs
Prereq: /sbin/install-info

%description
The GNU Compiler Collection, is a free software project that intends to
further the development of GNU compilers using an open development
environment. 

This package adds C support to the GNU Compiler Collection. It includes
support for most of the current C specification.  It does not include 
the standard C library.
separately.

%package -n cpp
Summary: GNU Compiler Collection - The C Preprocessor.
Group: Development/Languages
Prereq: /sbin/install-info

%description -n cpp
The GNU Compiler Collection, is a free software project that intends to
further the development of GNU compilers using an open development
environment. 

This package contains the C preprocessor. This is a 'macro processor'
used to automatically transform a program by the C compiler before 
actual compilation begins.

The C preprocessor provides four separate facilities:

* Inclusion of header files.  These are files of declarations to be
  substituted into your program.
* Macro expansion.  You can define 'macros', which are abbreviations for
  arbitrary fragments of C code, and then the C preprocessor will
  replace the macros with their definitions throughout the program.
* Conditional compilation.  Using special preprocessing directives,
  parts of a program can be included or excluded according to various
  conditions.
* Line control.  If you use a program to combine or rearrange source files
  into an intermediate file which is then compiled, line control can be used
  to inform the compiler where each source line originated.

%package c++
Summary: GNU Compiler Collection - The C++ Compiler
Group: Development/Languages
Requires: cpp = %{GCC_VERSION}
Requires: gcc = %{GCC_VERSION}
Requires: libstdc++ = %{STDC_VERSION}
Obsoletes: egcs-c++
Obsoletes: libg++-devel

%description c++
The GNU Compiler Collection, is a free software project that intends to
further the development of GNU compilers using an open development
environment. 

This package adds C++ support to the GNU Compiler Collection. It includes
support for most of the current C++ specification, including templates and
exception handling. It does include the static standard C++ library and C++
header files; the library for dynamically linking programs is available
separately.

%package objc
Summary: GNU Compiler Collection - The Objective C Compiler
Group: Development/Languages
Requires: cpp = %{GCC_VERSION}
Requires: gcc = %{GCC_VERSION}
Requires: gcc-c++ = %{GCC_VERSION}
Obsoletes: egcs-objc

%description objc
The GNU Compiler Collection, is a free software project that intends to
further the development of GNU compilers using an open development
environment. 

This package adds Objective-C support to the GNU Compiler Collection.  It
includes the Objective-C compiler and runtime that is required for running
Objective-C programs.

Objective C is an object-oriented derivative of the C language, much like
C++ but with more dynamic capabilities.

%package g77
Summary: GNU Compiler Collection - The Fortran 77 Compiler
Group: Development/Languages
Requires: cpp = %{GCC_VERSION}
Requires: gcc = %{GCC_VERSION}
Obsoletes: egcs-g77
Prereq: /sbin/install-info

%description g77
The GNU Compiler Collection, is a free software project that intends to
further the development of GNU compilers using an open development
environment. 

This package adds support for compiling Fortran 77 programs to the GNU
Compiler Collection.

%package java
Summary: GNU Compiler Collection - The Java Compiler
Group: Development/Languages
Requires: cpp = %{GCC_VERSION}
Requires: gcc = %{GCC_VERSION}
Requires: libgcj
Prereq: /sbin/install-info

%description java
The GNU Compiler Collection, is a free software project that intends to
further the development of GNU compilers using an open development
environment. 

This package adds support for compiling Java programs to the GNU Compiler
Collection.

%package -n libstdc++
Summary: GNU Compiler Collection - The C++ Library
Group: System Environment/Libraries
Obsoletes: libg++
Version: %{STDC_VERSION}

%description -n libstdc++
The GNU Compiler Collection, is a free software project that intends to
further the development of GNU compilers using an open development
environment. 

This package contains the GNU C++ runtime libraries.

%prep
%setup -q
%patch0 -p1
%patch1 -p1
%patch2 -p1
%patch3 -p0

mkdir compat
tar xzf %{SOURCE1} -C compat

# we don't need to build texinfo on a current Linux installation
rm -rf texinfo

%build
SRCDIR="$PWD"
rm -rf %{OBJDIR}
mkdir %{OBJDIR}
cd %{OBJDIR}

%ifarch armv4l
	CONFIG_FLAGS="--with-cpu=strongarm"
%else
	CONFIG_FLAGS=""
%endif

$SRCDIR/configure \
        --prefix=/usr $CONFIG_FLAGS \
	--enable-shared --enable-threads=posix \
	--without-cvs %{target_alias} \
        2>&1 | tee $SRCDIR/build.log

export PATH=$PATH:/sbin:/usr/sbin

make bootstrap-lean 2>&1 | tee -a $SRCDIR/build.log

# workaround a bug in the handling of MAKEINFO in gcc-2.95
find -name "*.info"|xargs rm -f
find -name "*.info-[1-9]*"|xargs rm -f
make info MAKEINFO="makeinfo --no-split" | tee -a $SRCDIR/build.log

%install
SRCDIR="$PWD"
rm -rf $RPM_BUILD_ROOT
cd %{OBJDIR}

export PATH=$PATH:/sbin:/usr/sbin

make install \
        prefix=$RPM_BUILD_ROOT/usr \
        MAKEINFO="makeinfo --no-split"

ln -sf gcc $RPM_BUILD_ROOT/usr/bin/cc
ln -sf g77 $RPM_BUILD_ROOT/usr/bin/f77

ln -sf g77.1 $RPM_BUILD_ROOT/usr/man/man1/f77.1
ln -sf cccp.1 $RPM_BUILD_ROOT/usr/man/man1/cpp.1

#FULLVER=$($RPM_BUILD_ROOT/usr/bin/gcc --version)
#FULLPATH=$(dirname $RPM_BUILD_ROOT/usr/lib/gcc-lib/$RPM_ARCH-*/$FULLVER/cc1)
#mkdir -p $RPM_BUILD_ROOT/lib
#ln -sf ../${FULLPATH##$RPM_BUILD_ROOT/}/cpp $RPM_BUILD_ROOT/lib/cpp

mkdir -p $RPM_BUILD_ROOT/lib
ln -sf /usr/bin/cpp $RPM_BUILD_ROOT/lib/cpp

gzip -n -9f $RPM_BUILD_ROOT/usr/info/*.info*

#install the compatibility libstdc++ library
[ -d $SRCDIR/compat/%{_target_cpu} ] && install -m 755 $SRCDIR/compat/%{_target_cpu}/* $RPM_BUILD_ROOT/usr/lib/

%clean
rm -rf $RPM_BUILD_ROOT

%post
/sbin/install-info --info-dir=/usr/info /usr/info/gcc.info.gz

%post -n cpp
/sbin/install-info --info-dir=/usr/info /usr/info/cpp.info.gz

%post g77
/sbin/install-info --info-dir=/usr/info /usr/info/g77.info.gz

%preun
if [ $1 = 0 ]; then
   /sbin/install-info --delete --info-dir=/usr/info /usr/info/gcc.info.gz
fi

%preun -n cpp
if [ $1 = 0 ]; then
   /sbin/install-info --delete --info-dir=/usr/info /usr/info/cpp.info.gz
fi

%preun g77
if [ $1 = 0 ]; then
   /sbin/install-info --delete --info-dir=/usr/info /usr/info/g77.info.gz
fi

%post -n libstdc++ -p /sbin/ldconfig

%postun -n libstdc++ -p /sbin/ldconfig

%files
%defattr(-,root,root)
%doc README* COPYING COPYING.LIB
%dir /usr/lib/gcc-lib
%dir /usr/lib/gcc-lib/%{target_alias}
%dir /usr/lib/gcc-lib/%{target_alias}/*
%dir /usr/lib/gcc-lib/%{target_alias}/*/include
%ifarch alpha
%dir /usr/lib/gcc-lib/%{target_alias}/*/ieee
%endif
%ifarch ppc
%dir /usr/lib/gcc-lib/%{target_alias}/*/nof
%endif
%ifarch armv4l armv3l
%dir /usr/lib/soft-float
%dir /usr/lib/gcc-lib/%{target_alias}/*/soft-float
%endif

/usr/bin/%{target_alias}-gcc
/usr/bin/gcov
/usr/bin/protoize
/usr/bin/unprotoize
/usr/bin/gcc
/usr/bin/cc
/usr/info/gcc*
/usr/man/man1/gcc.1
/usr/%{target_alias}
/usr/lib/gcc-lib/%{target_alias}/*/SYSCALLS.c.X
/usr/lib/gcc-lib/%{target_alias}/*/cc1
/usr/lib/gcc-lib/%{target_alias}/*/collect2
/usr/lib/gcc-lib/%{target_alias}/*/libgcc.a
%ifarch alpha ppc armv4l armv3l
/usr/lib/gcc-lib/%{target_alias}/*/*/libgcc.a
%endif
%ifnarch alpha
/usr/lib/gcc-lib/%{target_alias}/*/*crt*.o
%endif
/usr/lib/gcc-lib/%{target_alias}/*/include/README
/usr/lib/gcc-lib/%{target_alias}/*/include/float.h
/usr/lib/gcc-lib/%{target_alias}/*/include/iso646.h
/usr/lib/gcc-lib/%{target_alias}/*/include/limits.h
/usr/lib/gcc-lib/%{target_alias}/*/include/proto.h
/usr/lib/gcc-lib/%{target_alias}/*/include/stdarg.h
/usr/lib/gcc-lib/%{target_alias}/*/include/stdbool.h
/usr/lib/gcc-lib/%{target_alias}/*/include/stddef.h
/usr/lib/gcc-lib/%{target_alias}/*/include/syslimits.h
/usr/lib/gcc-lib/%{target_alias}/*/include/va-*.h
/usr/lib/gcc-lib/%{target_alias}/*/include/varargs.h
%ifarch ppc
/usr/lib/gcc-lib/%{target_alias}/*/include/*-asm.h
%endif

%files -n cpp
%defattr(-,root,root)
%dir /usr/lib/gcc-lib
%dir /usr/lib/gcc-lib/%{target_alias}
%dir /usr/lib/gcc-lib/%{target_alias}/*
%dir /usr/lib/gcc-lib/%{target_alias}/*/include
%ifarch alpha
%dir /usr/lib/gcc-lib/%{target_alias}/*/ieee
%endif
%ifarch ppc
%dir /usr/lib/gcc-lib/%{target_alias}/*/nof
%endif
%ifarch armv4l armv3l
%dir /usr/lib/soft-float
%dir /usr/lib/gcc-lib/%{target_alias}/*/soft-float
%endif

/lib/cpp
/usr/bin/cpp
/usr/info/cpp*
/usr/man/man1/cpp.1
/usr/man/man1/cccp.1
/usr/lib/gcc-lib/%{target_alias}/*/cpp
/usr/lib/gcc-lib/%{target_alias}/*/specs

%files c++
%defattr(-,root,root)
%dir /usr/lib/gcc-lib
%dir /usr/lib/gcc-lib/%{target_alias}
%dir /usr/lib/gcc-lib/%{target_alias}/*
%dir /usr/lib/gcc-lib/%{target_alias}/*/include
%ifarch alpha
%dir /usr/lib/gcc-lib/%{target_alias}/*/ieee
%endif
%ifarch ppc
%dir /usr/lib/gcc-lib/%{target_alias}/*/nof
%endif
%ifarch armv4l armv3l
%dir /usr/lib/soft-float
%dir /usr/lib/gcc-lib/%{target_alias}/*/soft-float
%endif

/usr/bin/g++
/usr/bin/c++
/usr/man/man1/g++.1
/usr/include/g++*
/usr/lib/libstdc++*.a
/usr/lib/libstdc++*.a.*
%ifarch alpha ppc armv4l armv3l
/usr/lib/*/libstdc++*.a
/usr/lib/*/libstdc++*.a.*
%endif
/usr/lib/gcc-lib/%{target_alias}/*/cc1plus
/usr/lib/gcc-lib/%{target_alias}/*/libstdc*
%ifarch alpha ppc armv4l armv3l
/usr/lib/gcc-lib/%{target_alias}/*/*/libstdc*
%endif
/usr/lib/gcc-lib/%{target_alias}/*/include/exception
/usr/lib/gcc-lib/%{target_alias}/*/include/new
/usr/lib/gcc-lib/%{target_alias}/*/include/typeinfo
/usr/lib/gcc-lib/%{target_alias}/*/include/new.h

%files objc
%defattr(-,root,root)
%dir /usr/lib/gcc-lib
%dir /usr/lib/gcc-lib/%{target_alias}
%dir /usr/lib/gcc-lib/%{target_alias}/*
%dir /usr/lib/gcc-lib/%{target_alias}/*/include
%ifarch alpha
%dir /usr/lib/gcc-lib/%{target_alias}/*/ieee
%endif
%ifarch ppc
%dir /usr/lib/gcc-lib/%{target_alias}/*/nof
%endif
%ifarch armv4l armv3l
%dir /usr/lib/soft-float
%dir /usr/lib/gcc-lib/%{target_alias}/*/soft-float
%endif

/usr/lib/gcc-lib/%{target_alias}/*/cc1obj
/usr/lib/gcc-lib/%{target_alias}/*/libobjc.a
/usr/lib/gcc-lib/%{target_alias}/*/include/objc
#libobjc.a is not multilibbed at the moment.
#%ifarch armv4l armv3l
#/usr/lib/*/libobjc.a
#/usr/lib/gcc-lib/%{target_alias}/*/*/libobjc.a
#%endif

%files g77
%defattr(-,root,root)
%dir /usr/lib/gcc-lib
%dir /usr/lib/gcc-lib/%{target_alias}
%dir /usr/lib/gcc-lib/%{target_alias}/*
%dir /usr/lib/gcc-lib/%{target_alias}/*/include
%ifarch alpha
%dir /usr/lib/gcc-lib/%{target_alias}/*/ieee
%endif
%ifarch ppc
%dir /usr/lib/gcc-lib/%{target_alias}/*/nof
%endif
%ifarch armv4l armv3l
%dir /usr/lib/soft-float
%dir /usr/lib/gcc-lib/%{target_alias}/*/soft-float
%endif

/usr/bin/g77
/usr/bin/f77
/usr/info/g77*
/usr/man/man1/g77.1
/usr/man/man1/f77.1
/usr/lib/gcc-lib/%{target_alias}/*/f771
/usr/lib/gcc-lib/%{target_alias}/*/libg2c.a
%ifarch alpha ppc armv4l armv3l
/usr/lib/gcc-lib/%{target_alias}/*/*/libg2c.a
%endif
/usr/lib/gcc-lib/%{target_alias}/*/include/g2c.h

%files java
%defattr(-,root,root)
%dir /usr/lib/gcc-lib
%dir /usr/lib/gcc-lib/%{target_alias}
%dir /usr/lib/gcc-lib/%{target_alias}/*
%dir /usr/lib/gcc-lib/%{target_alias}/*/include
%ifarch alpha
%dir /usr/lib/gcc-lib/%{target_alias}/*/ieee
%endif
%ifarch ppc
%dir /usr/lib/gcc-lib/%{target_alias}/*/nof
%endif
%ifarch armv4l armv3l
%dir /usr/lib/soft-float
%dir /usr/lib/gcc-lib/%{target_alias}/*/soft-float
%endif

/usr/bin/gcj
/usr/bin/gcjh
/usr/bin/jcf-dump
/usr/bin/jv-scan
/usr/lib/gcc-lib/%{target_alias}/*/jc1
/usr/lib/gcc-lib/%{target_alias}/*/jvgenmain

%files -n libstdc++
%defattr(-,root,root)
%ifarch alpha
%dir /usr/lib/ieee
%endif
%ifarch ppc
%dir /usr/lib/nof
%endif
%ifarch armv4l armv3l
%dir /usr/lib/soft-float
%dir /usr/lib/gcc-lib/%{target_alias}/*/soft-float
%endif

/usr/lib/libstdc++-3-libc*-%{STDC_VERSION}.so

%ifarch i386 sparc alpha
/usr/lib/libstdc++.so.2.8.0
/usr/lib/libstdc++-2-libc6.1-1-2.9.0.so
%endif
%ifarch i386 alpha
/usr/lib/libg++.so.2.7.2.8
/usr/lib/libstdc++.so.2.7.2.8
%endif
%ifarch alpha ppc
/usr/lib/*/libstdc++*.so
/usr/lib/*/libstdc++*.so.*
%endif
%ifarch armv4l 
/usr/lib/libstdc++.so.2.8.1.1
/usr/lib/libstdc++-2-libc6.1-1-2.9.0.so
%endif

%changelog
* Thu Sep 21 2000 Rod m. Stewart <stewart@netwinder.org>
- patch from Red Hat to address some libio changes wrt 
  glibc 2.1.x and glibc 2.1.9x/2.2
- Changed release to 1nw7.

* Wed May 24 2000 Rod m. Stewart <stewart@netwinder.org>
- use target_alias as host target name for configure
  (this is set to be %{_target_cpu}-unknown-linux-gnu)
- only configure with option --with-cpu=strongarm on armv4l archs
- add in armv3l use similar to armv4l for softfloat libraries
- Changed release to 1nw6.
- Do not package the compat libstdc++ libraries on armv3l

* Tue May 16 2000 Scott Bambrough <scottb@netwinder.org>
  - Added patch for arm.md file from Richard Earnshaw.
  - Changed release to 1nw5.

* Tue Mar 21 2000 Scott Bambrough <scottb@netwinder.org>
  - Updated modified version of Bryce McKinlay's backport of the
    Java compiler from the mainline CVS sources.
  - Changed ARM patch to remove my java patches, now in java patch
  - Changed requires for binutils to >= 2.9.5.0.19 as suggested
    by Robert Harley 

* Sat Feb 12 2000 Scott Bambrough <scottb@netwinder.org>
  - Documented where patches came from.
  - Added scheduling patch fix from Jeff Law.
  - Changed release to 1nw4

* Sun Feb 06 2000 Scott Bambrough <scottb@netwinder.org>
  - Added patch to fix gcjh double generation on ARM
  - Changed release to 1nw3

* Thu Jan 20 2000 Scott Bambrough <scottb@netwinder.org>
  - Added Phil Blundell's degenerate stm/ldm patch
  - Added Phil Blundell's clobber fix for CLEAR_INSN_CACHE
  - Updated modified version of Bryce McKinlay's backport of the
    Java compiler from the mainline CVS sources.
  - Changed /lib/cpp symlink to point to /usr/bin/cpp; currently points
    at one in compiler directory; breaks Motif build; reported by MetroLink
  - Changed requires for binutils to 2.9.5.0.19 as suggested by Rod Stewart 
  - Changed release to 1nw2

* Tue Dec 14 1999 Scott Bambrough <scottb@netwinder.org>
  - Updated to gcc-2.95.2 release tarball
  - Added modified version of Phil Blundell's patches for ARM
  - Added modified version of HJ Lu's patches for libobjc
  - Added modified version of Bryce McKinlay's backport of the
    Java compiler from the mainline CVS sources.
  - Removed /usr/bin/c++filt from the C++ file list, use binutils version
  - Added requires for libfloat
  - Changed release to 1nw1

* Wed Oct 20 1999 Scott Bambrough <scottb@netwinder.org>
  - Added requires for libfloat if arch=armv4l
  - Changed release to 1nw6

* Tue Sep 21 1999 Scott Bambrough <scottb@netwinder.org>
  - Added libstdc++.so.2.8.1.1 to compatibility tarball
  - Added libstdc++-2-libc6-1-1-2.9.0.so to compatibility tarball
  - Added the above files to package libstdc++.

* Mon Sep 20 1999 Scott Bambrough <scottb@netwinder.org>
  - Fixed patch to get Java double values correct for ARM.
  - Changed release to 1nw5, release 1nw4 never released

* Sun Sep 19 1999 Scott Bambrough <scottb@netwinder.org>
  - Added patch to generate gcj/array.h rather than java-array.h
  - Modified spec file to remove Obsoletes clauses for packages
    in this RPM.
  - Changed release to 1nw4, release 1nw3 never released

* Sat Sep 18 1999 Scott Bambrough <scottb@netwinder.org>
  - Added patch to get Java double values correct for ARM.
  - Added patch to add FUNCTION_PROFILER to linux-elf.h in an
    attempt to get profiling working again.
  - Added patch to to fix selective3 failure in linker testsuite
  - Modified spec file to add armv4l soft-float multilib pieces
  - Added %dir /usr/lib/soft-float to all packages file lists
  - Added %dir /usr/lib/gcc-lib/%{target_alias}/*/soft-float
    to all packages file lists
  - Changed release to 1nw3

* Thu Sep 02 1999 Scott Bambrough <scottb@netwinder.org>
  - Added patch to add NO_IMPLICIT_EXTERN_C to linux-elf.h.  This 
    will allow KDE to build correctly.  The compiler will no longer
    assume headers in the /usr/include path require an implicit
    extern "C" around the header during C++ compiles.
  - Added patch to prevent volatile range tests from being optimized 
    away.  This should prevent the probe code from getting optimized
    away in the ARM Linux kernel.
  - Changed release to 1nw2.

* Thu Aug 19 1999 Scott Bambrough <scottb@netwinder.org>
  - Updated to gcc-2.95.1

* Wed Aug 11 1999 Scott Bambrough <scottb@netwinder.org>
  - Added ARM patch from Phil Blundell.
  - changed release

* Wed Aug  4 1999 H.J. Lu <hjl@lucon.org>
  - Add some patches.
  - Add compatibility changes.

* Fri Jul 16 1999 Franz Sirl <Franz.Sirl-kernel@lauterbach.com>
  - overhauled the RH-6.0 spec (done by Cristian Gafton <gafton@redhat.com>)
    to fit gcc-2.95
